@@ -7,6 +7,7 @@ import feedparser
 import argparse
 import hashlib
 import urllib
+import base64
 import redis
 import types
 import json
@@ -426,10 +427,51 @@ class TRMaid(object):
             self.msg('handling feed %s...' % feed_url[:40])
             self.add_links('rss', feed_url, links(feed_url), rss=True)
 
+    def get_torrent_files(self, path):
+        if not os.path.isdir(path):
+            self.msg('! not an folder: %s' % path)
+        else:
+            for file in os.listdir(path):
+                if not file.endswith('.torrent'):
+                    continue
+                yield os.path.join(path, file)
+
+    def handle_watch_folders(self):
+        folders = self.conf.get('watch-folders')
+        if not folders:
+            return
+        for folder, conf in folders.iteritems():
+            self.msg('handling folder %s' % folder)
+            ids = []
+            add_req = self.get_torrent_add_args('watch-folders', folder)
+            for file in self.get_torrent_files(folder):
+                content = base64.b64encode(open(file).read())
+                add_req['metainfo'] = content
+                res = self.verbose_func(self.rpc.torrent_add, add_req)
+                result = res.get('result', '')
+
+                if result in ['success', 'duplicate torrent']:
+                    os.rename(file, file + '.added')
+
+                if result != 'success':
+                    continue
+
+                arguments = res.get('arguments', {})
+
+                if arguments.get('torrent-added', None):
+                    ids.append(arguments['torrent-added']['id'])
+
+            if ids:
+                set_req = self.get_torrent_set_args('watch-folders', folder)
+                if set_req:
+                    set_req['ids'] = ids
+                    self.verbose_func(self.rpc.torrent_set, set_req)
+
     def run(self):
         self.handle_session()
         self.handle_rss()
         self.handle_redis()
+        self.handle_watch_folders()
         self.handle_all_torrents()
 
 
